from hm_pyhelper.exceptions import ECCMalfunctionException, \
    MinerFailedToFetchMacAddress
from hm_pyhelper.lock_singleton import ResourceBusyError
from hm_pyhelper.miner_param import get_gateway_mfr_test_result, \
    retry_get_region, await_spi_available, \
    provision_key, \
    did_gateway_mfr_test_result_include_miner_key_pass, \
    get_mac_address
import unittest
from unittest.mock import mock_open, patch
import pytest

ALL_PASS_GATEWAY_MFR_TESTS = [
    {
        "output": "ok",
        "result": "pass",
        "test": "serial"
    },
    {
        "output": "ok",
        "result": "pass",
        "test": "zone_locked(data)"
    },
    {
        "output": "ok",
        "result": "pass",
        "test": "zone_locked(config)"
    },
    {
        "output": "ok",
        "result": "pass",
        "test": "slot_config(0..=15, ecc)"
    },
    {
        "output": "ok",
        "result": "pass",
        "test": "key_config(0..=15, ecc)"
    },
    {
        "output": "ok",
        "result": "pass",
        "test": "miner_key(0)"
    }
]

NONE_PASS_GATEWAY_MFR_TESTS = [
    {
        "output": "timeout/retry error",
        "result": "fail",
        "test": "serial"
    },
    {
        "output": "timeout/retry error",
        "result": "fail",
        "test": "zone_locked(data)"
    },
    {
        "output": "timeout/retry error",
        "result": "fail",
        "test": "zone_locked(config)"
    },
    {
        "output": "timeout/retry error",
        "result": "fail",
        "test": "slot_config(0..=15, ecc)"
    },
    {
        "output": "timeout/retry error",
        "result": "fail",
        "test": "key_config(0..=15, ecc)"
    },
    {
        "output": "timeout/retry error",
        "result": "fail",
        "test": "miner_key(0)"
    }
  ]


class GatewayMfrProvisionMock:
    stderr = "example error"

    stdout = """{
          "provision": "example"
        }"""


@patch.dict('os.environ', {"BALENA_DEVICE_TYPE": "raspberrypi3-64"})
class TestMinerParam(unittest.TestCase):
    @patch(
            'hm_pyhelper.miner_param.get_gateway_mfr_test_result',
            return_value={
                "result": "pass",
                "tests": ALL_PASS_GATEWAY_MFR_TESTS
                }
    )
    def test_provision_key_all_passed(self, get_gateway_mfr_test_result):
        self.assertTrue(provision_key())

    @patch('subprocess.run', return_value=GatewayMfrProvisionMock())
    @patch(
            'hm_pyhelper.miner_param.get_gateway_mfr_test_result',
            return_value={
                "result": "fail",
                "tests": NONE_PASS_GATEWAY_MFR_TESTS
            }
    )
    def test_provision_key_none_passed(
            self,
            get_gateway_mfr_test_result,
            subprocess_run
    ):
        self.assertTrue(provision_key())

    @patch(
            'hm_pyhelper.miner_param.get_gateway_mfr_test_result',
            return_value={
                "result": "fail",
                "tests": NONE_PASS_GATEWAY_MFR_TESTS
            }
    )
    def test_did_gateway_mfr_test_result_include_miner_key_fail(
            self,
            get_gateway_mfr_test_result
    ):
        self.assertFalse(
                did_gateway_mfr_test_result_include_miner_key_pass(
                    get_gateway_mfr_test_result
                    )
        )

    def test_did_gateway_mfr_test_result_include_miner_key_pass(self):
        get_gateway_mfr_test_result = {
            "result": "fail",
            "tests": [
                {
                    "output": "timeout/retry error",
                    "result": "fail",
                    "test": "serial"
                },
                {
                    "output": "timeout/retry error",
                    "result": "fail",
                    "test": "zone_locked(data)"
                },
                {
                    "output": "timeout/retry error",
                    "result": "fail",
                    "test": "zone_locked(config)"
                },
                {
                    "output": "timeout/retry error",
                    "result": "fail",
                    "test": "slot_config(0..=15, ecc)"
                },
                {
                    "output": "timeout/retry error",
                    "result": "fail",
                    "test": "key_config(0..=15, ecc)"
                },
                {
                    "output": "ok",
                    "result": "pass",
                    "test": "miner_key(0)"
                }
            ]
        }
        self.assertTrue(
                did_gateway_mfr_test_result_include_miner_key_pass(
                    get_gateway_mfr_test_result
                    )
        )

    def test_get_region_from_override(self):
        self.assertEqual(retry_get_region("foo", "bar/"), 'foo')

    @patch("builtins.open", new_callable=mock_open, read_data="ZZ111\n")
    def test_get_region_from_miner(self, _):
        self.assertEqual(retry_get_region(False, "foo/"), 'ZZ111')  # noqa: E501

    @patch("os.path.exists", return_value=True)
    def test_is_spi_available(self, _):
        self.assertTrue(await_spi_available("spiXY.Z"))

    def test_error_mac_address(self):
        with pytest.raises(MinerFailedToFetchMacAddress):
            get_mac_address("test/path")

    @patch(
      'hm_pyhelper.miner_param.run_gateway_mfr',
      side_effect=FileNotFoundError("File Not Found Error"))
    def test_filenotfound_exception(self, mock):
        with pytest.raises(FileNotFoundError):
            get_gateway_mfr_test_result()

    @patch(
      'hm_pyhelper.miner_param.run_gateway_mfr',
      side_effect=ECCMalfunctionException("File Not Found Error"))
    def test_eccmalfunction_exception(self, mock):
        with pytest.raises(ECCMalfunctionException):
            get_gateway_mfr_test_result()

    @patch(
      'hm_pyhelper.miner_param.run_gateway_mfr',
      side_effect=ResourceBusyError("Resource Busy Error"))
    def test_resourcebusy_exception(self, mock):
        with pytest.raises(ResourceBusyError):
            get_gateway_mfr_test_result()



import unittest
import json

from hm_pyhelper.diagnostics import Diagnostic, DiagnosticsReport
from hm_pyhelper.diagnostics.diagnostics_report import \
                                    DIAGNOSTICS_PASSED_KEY, \
                                    DIAGNOSTICS_ERRORS_KEY


class TestDiagnostic(unittest.TestCase):
    def test_record_result(self):
        diagnostic = Diagnostic('key', 'friendly_name')
        diagnostics_report = DiagnosticsReport()
        diagnostics_report.record_result('foo', diagnostic)

        self.assertDictEqual(diagnostics_report, {
            DIAGNOSTICS_PASSED_KEY: False,
            DIAGNOSTICS_ERRORS_KEY: [],
            'key': 'foo',
            'friendly_name': 'foo'
        })

    def test_record_failure(self):
        diagnostic = Diagnostic('key', 'friendly_name')
        diagnostics_report = DiagnosticsReport()
        diagnostics_report.record_failure('foo', diagnostic)

        self.assertDictEqual(diagnostics_report, {
            DIAGNOSTICS_PASSED_KEY: False,
            DIAGNOSTICS_ERRORS_KEY: ['key'],
            'key': 'foo',
            'friendly_name': 'foo'
        })

    def test_deserialization(self):
        report_str = '{"diagnostics_passed": false, "errors": ["ECC"], "ECC": false, "foo": "bar"}'  # noqa E501
        report_dict = json.loads(report_str)
        diagnostics_report = DiagnosticsReport(**report_dict)

        self.assertDictEqual(diagnostics_report, {
            DIAGNOSTICS_PASSED_KEY: False,
            DIAGNOSTICS_ERRORS_KEY: ['ECC'],
            'ECC': False,
            'foo': 'bar'
        })

    def test_get_error_messages(self):
        diagnostic1 = Diagnostic('key1', 'friendly_name')
        diagnostic2 = Diagnostic('key2', 'friendly_name')
        diagnostics_report = DiagnosticsReport()
        diagnostics_report.record_failure('Error1', diagnostic1)
        diagnostics_report.record_failure('Error2', diagnostic2)

        actual_msgs = diagnostics_report.get_error_messages()
        expected_msgs = "key1 Error: Error1\nkey2 Error: Error2"
        self.assertEqual(actual_msgs, expected_msgs)

    def test_get_report_subset(self):
        diagnostics_report = DiagnosticsReport()
        diagnostics_report['VA'] = 'NEBHNT-IN1'
        diagnostics_report['foo'] = 'bar'

        self.assertDictEqual(diagnostics_report.get_report_subset(["VA"]), {
            'VA': 'NEBHNT-IN1'
        })



import os


def is_rockpi():
    return 'rockpi-4b-rk3399' in os.getenv('BALENA_DEVICE_TYPE')


def is_raspberry_pi():
    """
    Pulled from
    https://www.balena.io/docs/reference/base-images/devicetypes/
    """
    device_type = os.getenv('BALENA_DEVICE_TYPE')
    device_type_match = [
        'raspberry-pi2',
        'raspberrypi3',
        'raspberrypi3-64',
        'raspberrypi4-64',
        'nebra-hnt',
        'raspberrypicm4-ioboard'
    ]

    for device in device_type_match:
        if device in device_type:
            return True
    return False


variant_definitions = {
    # Nebra Indoor Hotspot Gen1
    'NEBHNT-IN1': {
        'FRIENDLY': 'Nebra Indoor Hotspot Gen 1',
        'APPNAME': 'Indoor',
        'SPIBUS': 'spidev1.2',
        'RESET': 38,
        'MAC': 'eth0',
        'STATUS': 25,
        'BUTTON': 26,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Nebra Indoor Hotspot, Old identifier
    'Indoor': {
        'FRIENDLY': 'Nebra Indoor Hotspot Gen 1',
        'APPNAME': 'Indoor',
        'SPIBUS': 'spidev1.2',
        'RESET': 38,
        'MAC': 'eth0',
        'STATUS': 25,
        'BUTTON': 26,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Nebra Outdoor Hotspot Gen1
    'NEBHNT-OUT1': {
        'FRIENDLY': 'Nebra Outdoor Hotspot Gen 1',
        'APPNAME': 'Outdoor',
        'SPIBUS': 'spidev1.2',
        'RESET': 38,
        'MAC': 'eth0',
        'STATUS': 25,
        'BUTTON': 24,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': True
        },

    # Nebra Outdoor Hotspot Old Identifier
    'Outdoor': {
        'FRIENDLY': 'Nebra Outdoor Hotspot Gen 1',
        'APPNAME': 'Outdoor',
        'SPIBUS': 'spidev1.2',
        'RESET': 38,
        'MAC': 'eth0',
        'STATUS': 25,
        'BUTTON': 24,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': True
        },

    # Nebra Pi 0 Light Hotspot SPI Ethernet
    'NEBHNT-LGT-ZS': {
        'FRIENDLY': 'Nebra Pi 0 Light Hotspot SE',
        'APPNAME': 'Pi 0 Light',
        'SPIBUS': 'spidev1.2',
        'RESET': 22,
        'MAC': 'wlan0',
        'STATUS': 24,
        'BUTTON': 23,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },
    # Nebra Pi 0 Light Hotspot USB Ethernet
    'NEBHNT-LGT-ZX': {
        'FRIENDLY': 'Nebra Pi 0 Light Hotspot XE',
        'APPNAME': 'Pi 0 Light',
        'SPIBUS': 'spidev1.2',
        'RESET': 22,
        'MAC': 'wlan0',
        'STATUS': 24,
        'BUTTON': 23,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Nebra Beaglebone Light Hotspot
    'NEBHNT-BBB': {
        'FRIENDLY': 'Nebra Beaglebone Light Hotspot',
        'APPNAME': 'Beaglebone Light',
        'SPIBUS': 'spidev1.0',
        'RESET': 60,
        'MAC': 'eth0',
        'STATUS': 31,
        'BUTTON': 30,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Nebra Pocket Beagle Light Hotspot
    'NEBHNT-PBB': {
        'FRIENDLY': 'Nebra Pocket Beagle Light Hotspot',
        'APPNAME': 'PB Light',
        'SPIBUS': 'spidev1.2',
        'RESET': 60,
        'MAC': 'wlan0',
        'STATUS': 31,
        'BUTTON': 30,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Nebra Hotspot HAT ROCK Pi 4 Indoor
    'NEBHNT-HHRK4': {
        'FRIENDLY': 'Nebra ROCK Pi 4 Indoor',
        'APPNAME': 'ROCK Pi',
        'SPIBUS': 'spidev32766.0',
        'RESET': 149,
        'MAC': 'eth0',
        'STATUS': 156,
        'BUTTON': 154,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Nebra Hotspot HAT ROCK Pi 4 Outdoor
    'NEBHNT-HHRK4-OUT': {
        'FRIENDLY': 'Nebra ROCK Pi 4 Outdoor',
        'APPNAME': 'ROCK Pi',
        'SPIBUS': 'spidev32766.0',
        'RESET': 149,
        'MAC': 'eth0',
        'STATUS': 156,
        'BUTTON': 154,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': True
        },

    # Nebra Hotspot HAT RPi 3/4 Full
    'NEBHNT-HHRPI': {
        'FRIENDLY': 'Nebra Hotspot HAT RPi',
        'APPNAME': 'RPi',
        'SPIBUS': 'spidev0.0',
        'RESET': 22,
        'MAC': 'eth0',
        'STATUS': 24,
        'BUTTON': 23,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Nebra Hotspot HAT RPi Light
    'NEBHNT-HHRPL': {
        'FRIENDLY': 'Nebra Hotspot HAT RPi Light',
        'APPNAME': 'Light RPi',
        'SPIBUS': 'spidev0.0',
        'RESET': 22,
        'MAC': 'eth0',
        'STATUS': 24,
        'BUTTON': 23,
        'ECCOB': True,
        'TYPE': 'Light',
        'CELLULAR': False
        },

    # Nebra Hotspot HAT Tinkerboard 1
    'NEBHNT-HHTK': {
        'FRIENDLY': 'Nebra Hotspot HAT Tinkerboard Light',
        'APPNAME': 'Tinkerboard Light',
        'SPIBUS': 'spidev2.0',
        'RESET': 167,
        'MAC': 'eth0',
        'STATUS': 163,
        'BUTTON': 162,
        'ECCOB': True,
        'TYPE': 'Light',
        'CELLULAR': False
        },

    # Nebra Hotspot HAT Tinkerboard 2
    'NEBHNT-HHTK2': {
        'FRIENDLY': 'Nebra Hotspot HAT Tinkerboard 2',
        'APPNAME': 'Tinkerboard',
        'SPIBUS': 'spidev2.0',
        'RESET': 167,
        'MAC': 'eth0',
        'STATUS': 163,
        'BUTTON': 162,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # RAKwireless Hotspot Miner
    'COMP-RAKHM': {
        'FRIENDLY': 'RAK Hotspot',
        'SPIBUS': 'spidev0.0',
        'RESET': 17,
        'MAC': 'wlan0',
        'STATUS': 20,
        'BUTTON': 21,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # Helium Hotspot
    'COMP-HELIUM': {
        'FRIENDLY': 'Helium Hotspot',
        'SPIBUS': 'spidev0.0',
        'RESET': 17,
        'MAC': 'wlan0',
        'STATUS': 20,
        'BUTTON': 21,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # SenseCAP M1 Hotspot
    'COMP-SENSECAPM1': {
        'FRIENDLY': 'SenseCAP M1',
        'SPIBUS': 'spidev0.0',
        'RESET': 17,
        'MAC': 'wlan0',
        'STATUS': 20,
        'BUTTON': 21,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False
        },

    # DIY Pi Supply Hotspot HAT
    'DIY-PISLGH': {
        'FRIENDLY': 'DIY Pi Supply Hotspot HAT',
        'SPIBUS': 'spidev0.0',
        'RESET': 22,
        'MAC': 'eth0',
        'STATUS': 20,
        'BUTTON': 21,
        'ECCOB': False,
        'TYPE': 'Light',
        'CELLULAR': False
        },

    # Nebra Indoor Hotspot
    'DIY-RAK2287': {
        'FRIENDLY': 'DIY RAK2247/RAK2287 HAT',
        'SPIBUS': 'spidev0.0',
        'RESET': 17,
        'MAC': 'eth0',
        'STATUS': 20,
        'BUTTON': 21,
        'ECCOB': False,
        'TYPE': 'Light',
        'CELLULAR': False
        }
}



import unittest
import mock

from hm_pyhelper.miner_json_rpc import MinerClient
from hm_pyhelper.miner_json_rpc.exceptions import MinerRegionUnset
from hm_pyhelper.miner_json_rpc.exceptions import MinerMalformedURL
from hm_pyhelper.miner_json_rpc.exceptions import MinerConnectionError


BASE_URL = 'http://helium-miner:4467'


class Result(object):
    def __init__(self, result={'my': 'data'}):
        self.result = result


class Response(object):
    def __init__(self, data=Result()):
        self.data = data


class TestMinerJSONRPC(unittest.TestCase):

    def test_instantiation(self):
        client = MinerClient()
        self.assertIsInstance(client, MinerClient)
        self.assertEqual(client.url, BASE_URL)

    def test_malformed_url(self):
        client = MinerClient(url='fakeurl')

        exception_raised = False
        exception_type = None
        try:
            client.get_height()
        except Exception as exc:
            exception_raised = True
            exception_type = exc

        self.assertTrue(exception_raised)
        self.assertIsInstance(exception_type, MinerMalformedURL)

    def test_connection_error(self):
        client = MinerClient(url='http://notarealminer:9999')

        exception_raised = False
        exception_type = None
        try:
            client.get_height()
        except Exception as exc:
            exception_raised = True
            exception_type = exc

        self.assertTrue(exception_raised)
        self.assertIsInstance(exception_type, MinerConnectionError)

    @mock.patch('hm_pyhelper.miner_json_rpc.client.request')
    def test_get_height(self, mock_json_rpc_client):
        mock_json_rpc_client.return_value = Response(
            data=Result(
                result={'epoch': 25612, 'height': 993640}
            )
        )
        client = MinerClient()
        result = client.get_height()
        mock_json_rpc_client.assert_called_with(
            BASE_URL,
            'info_height'
        )
        self.assertEqual(result, {'epoch': 25612, 'height': 993640})

    @mock.patch('hm_pyhelper.miner_json_rpc.client.request')
    def test_get_region_not_asserted(self, mock_json_rpc_client):
        mock_json_rpc_client.return_value = Response(
            data=Result(
                result={'region': None}
            )
        )
        client = MinerClient()
        exception_raised = False
        exception_type = None

        try:
            client.get_region()
        except Exception as exc:
            exception_raised = True
            exception_type = exc

        self.assertTrue(exception_raised)
        self.assertIsInstance(exception_type, MinerRegionUnset)

    @mock.patch('hm_pyhelper.miner_json_rpc.client.request')
    def test_get_region(self, mock_json_rpc_client):
        mock_json_rpc_client.return_value = Response(
            data=Result(
                result={'region': 'EU868'}
            )
        )
        client = MinerClient()
        result = client.get_region()
        mock_json_rpc_client.assert_called_with(
            BASE_URL,
            'info_region'
        )
        self.assertEqual(result, {'region': 'EU868'})

    @mock.patch('hm_pyhelper.miner_json_rpc.client.request')
    def test_get_summary(self, mock_json_rpc_client):
        fw_version_err = ("cat: can't open '/etc/lsb_release': ",
                          "No such file or directory\n")
        summary = {
            'block_age': 1136610,
            'epoch': 25612,
            'firmware_version': fw_version_err,
            'gateway_details': 'undefined',
            'height': 993640,
            'mac_addresses': [
                {'eth0': '0242AC110002'},
                {'ip6tnl0': '00000000000000000000000000000000'},
                {'tunl0': '00000000'},
                {'lo': '000000000000'}
            ],
            'name': 'scruffy-chocolate-shell',
            'peer_book_entry_count': 3,
            'sync_height': 993640,
            'uptime': 144,
            'version': 10010005
        }
        mock_json_rpc_client.return_value = Response(
            data=Result(
                result=summary
            )
        )
        client = MinerClient()
        result = client.get_summary()
        mock_json_rpc_client.assert_called_with(
            BASE_URL,
            'info_summary'
        )
        self.assertEqual(result, summary)

    @mock.patch('hm_pyhelper.miner_json_rpc.client.request')
    def test_get_peer_addr(self, mock_json_rpc_client):
        peer_addr = '/p2p/11jr2kMp1bZvSC6pd3XkNvs9Q43qCgEzxRwV6vpuqXanC5UcLEs'
        mock_json_rpc_client.return_value = Response(
            data=Result(
                result={'peer_addr': peer_addr}
            )
        )
        client = MinerClient()
        result = client.get_peer_addr()
        mock_json_rpc_client.assert_called_with(
            BASE_URL,
            'peer_addr'
        )
        self.assertEqual(result, {'peer_addr': peer_addr})

    @mock.patch('hm_pyhelper.miner_json_rpc.client.request')
    def test_get_peer_book(self, mock_json_rpc_client):
        mock_json_rpc_client.return_value = Response(
            data=Result(
                result=[]
            )
        )
        client = MinerClient()
        result = client.get_peer_book()
        mock_json_rpc_client.assert_called_with(
            BASE_URL,
            'peer_book',
            addr='self'
        )
        self.assertEqual(result, [])

    @mock.patch('hm_pyhelper.miner_json_rpc.client.request')
    def test_get_firmware_version(self, mock_json_rpc_client):
        firmware_version = '2021.10.18.0'
        summary = {
            'block_age': 1136610,
            'epoch': 25612,
            'firmware_version': firmware_version,
            'gateway_details': 'undefined',
            'height': 993640,
            'mac_addresses': [
                {'eth0': '0242AC110002'},
                {'ip6tnl0': '00000000000000000000000000000000'},
                {'tunl0': '00000000'},
                {'lo': '000000000000'}
            ],
            'name': 'scruffy-chocolate-shell',
            'peer_book_entry_count': 3,
            'sync_height': 993640,
            'uptime': 144,
            'version': 10010005
        }
        mock_json_rpc_client.return_value = Response(
            data=Result(
                result=summary
            )
        )
        client = MinerClient()
        result = client.get_firmware_version()
        mock_json_rpc_client.assert_called_with(
            BASE_URL,
            'info_summary'
        )
        self.assertEqual(result, firmware_version)
